package org.greenstone.atea;


import com.mongodb.client.MongoCollection; 
import com.mongodb.client.MongoDatabase; 
import com.mongodb.MongoClient; 
import com.mongodb.MongoCredential;  

import org.bson.Document;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Properties;

import org.apache.log4j.Logger;


/** 
 * https://www.tutorialspoint.com/mongodb/mongodb_java.htm
 *
 * TO COMPILE:
 *    maori-lang-detection/src$
 *       javac -cp ".:../conf:../lib/*" org/greenstone/atea/MongoDBAccess.java
 *
 * TO RUN:
 *       java -cp ".:../conf:../lib/*" org.greenstone.atea.MongoDBAccess
 *
 * Manually connecting to mongodb from client:
 *    mongo 'mongodb://mongodb.cms.waikato.ac.nz:27017' -u USERNAME -p
 * Then after connecting with pwd, type:
 *    use DBNAME
 *
 * Or connect to mongodb and specify db in one statement:
 *    mongo 'mongodb://mongodb.cms.waikato.ac.nz:27017/DBNAME?authSource=admin' -u USERNAME -p
 *
 * Some links:
 *   - https://stackoverflow.com/questions/19938153/do-i-need-to-explicitly-close-connection
 *   - https://docs.mongodb.com/manual/reference/glossary/ (particularly "collection")
 *   - https://tecadmin.net/tutorial/mongodb/drop-collection/
 * IMPORTANT LINK:
 *   - https://www.mongodb.com/blog/post/6-rules-of-thumb-for-mongodb-schema-design-part-1
 *
 */
public class MongoDBAccess {

    private static Logger logger = Logger.getLogger(org.greenstone.atea.MongoDBAccess.class.getName());
    
    String HOST = "localhost";
    int PORT = 27017; // mongodb port
    String PROPS_FILENAME = "config.properties";
    String DB_NAME = "ateacrawldata";
    
    private String USERNAME;
    private String PASSWORD;


    private MongoClient mongo = null;
    private MongoDatabase database = null;
    
    public MongoDBAccess() throws Exception {
	boolean success = false;
	
	// Read in the username and password from our props file
	Properties props = new Properties();

	//File propsFile = new File(PROPS_FILENAME);
	//logger.debug("*** Conf props filename: " + propsFile.getAbsolutePath());
	try {
	    props.load(getClass().getClassLoader().getResourceAsStream(PROPS_FILENAME));	    
	} catch(Exception e) {
	    logger.error(e);
	}
	
	
	USERNAME = props.getProperty("mongodb.user", "");
	if(USERNAME.equals("")) {
	    USERNAME = "root";
	    logger.warn("WARNING: No sensible value for mongodb.user specified in " + PROPS_FILENAME + ". Attempting to use: " + USERNAME);
			}
	PASSWORD = props.getProperty("mongodb.pwd");

	logger.debug("Got pwd: " + PASSWORD);
	
	if(PASSWORD != null && PASSWORD.equals("CHANGEME")) {
	    
	    success = false;
	    throw new Exception("************ FATAL ERROR: Change DB password in properties file " + PROPS_FILENAME);	    
	}

	HOST = props.getProperty("mongodb.host", HOST);
	String port = props.getProperty("mongodb.port", Integer.toString(PORT));
	PORT = Integer.parseInt(port);
	DB_NAME = props.getProperty("mongodb.dbname", DB_NAME);

	logger.info("Connecting to mongodb with:");
	logger.info(" - host:    " + HOST);
	logger.info(" - port:    " + PORT);
	logger.info(" - user:    " + USERNAME);
	logger.info(" - db name: " + DB_NAME);	
    }

    /** 
     * Since we have only a single MongoClient, don't need to call close/disconnect on it as per
     * https://stackoverflow.com/questions/19938153/do-i-need-to-explicitly-close-connection
     */
    public void connectToDB() throws Exception {
	// Creating a Mongo client 
	mongo = new MongoClient( HOST, PORT ); 
	
	// Creating Credentials 
	MongoCredential credential; 
	credential = MongoCredential.createCredential(USERNAME, DB_NAME, PASSWORD.toCharArray()); 
	System.out.println("Connected to the database successfully");  
	
	// Accessing the database 
	database = mongo.getDatabase(DB_NAME); 
	logger.info("Credentials: "+ credential);
    }
    

    /*
    public void insertDocument() {
	MongoCollection<Document> collection = this.database.getCollection("sampleCollection");
    }
    */

    // TODO:
    // In the database, need to ensure we have else
    // create collection (table in RDBMS) websites, create collection webpages.
    // The webpages collection will have sentences embedded based on my decisions from
    // reading the series
    // https://www.mongodb.com/blog/post/6-rules-of-thumb-for-mongodb-schema-design-part-1    
    // Then need functions:
    // insertWebsiteDocument()
    // insertWebpageDocument()
    
    public static void main(String args[]) {
	try {
	    MongoDBAccess mongodbCon = new MongoDBAccess();
	    mongodbCon.connectToDB();
	    //mongodbCon.insertDocument();
	}catch(Exception e) {
	    e.printStackTrace();
	}
    }
}
